@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #include "PdfObjectStreamParserObject.h"
@@ -48,8 +61,8 @@ PdfObjectStreamParserObject::~PdfObjectStreamParserObject()
 
 void PdfObjectStreamParserObject::Parse(ObjectIdList const & list)
 {
-    long long lNum   = m_pParser->GetDictionary().GetKeyAsLong( "N", 0 );
-    long long lFirst = m_pParser->GetDictionary().GetKeyAsLong( "First", 0 );
+    pdf_int64 lNum   = m_pParser->GetDictionary().GetKeyAsLong( "N", 0 );
+    pdf_int64 lFirst = m_pParser->GetDictionary().GetKeyAsLong( "First", 0 );
     
     char* pBuffer;
     pdf_long lBufferLen;
@@ -57,29 +70,29 @@ void PdfObjectStreamParserObject::Parse(ObjectIdList const & list)
 
     try {
         this->ReadObjectsFromStream( pBuffer, lBufferLen, lNum, lFirst, list );
-        free( pBuffer );
+        podofo_free( pBuffer );
 
         // the object stream is not needed anymore in the final PDF
         delete m_vecObjects->RemoveObject( m_pParser->Reference() );
         m_pParser = NULL;
 
-    } catch( const PdfError & rError ) {
-        free( pBuffer );
+    } catch( PdfError & rError ) {
+        podofo_free( pBuffer );
         throw rError;
     }
 }
 
-void PdfObjectStreamParserObject::ReadObjectsFromStream( char* pBuffer, pdf_long lBufferLen, long long lNum, long long lFirst, ObjectIdList const & list)
+void PdfObjectStreamParserObject::ReadObjectsFromStream( char* pBuffer, pdf_long lBufferLen, pdf_int64 lNum, pdf_int64 lFirst, ObjectIdList const & list)
 {
     PdfRefCountedInputDevice device( pBuffer, lBufferLen );
     PdfTokenizer             tokenizer( device, m_buffer );
     PdfVariant               var;
     int                      i = 0;
 
-    while( static_cast<long long>(i) < lNum )
+    while( static_cast<pdf_int64>(i) < lNum )
     {
-        const long long lObj     = tokenizer.GetNextNumber();
-        const long long lOff     = tokenizer.GetNextNumber();
+        const pdf_int64 lObj     = tokenizer.GetNextNumber();
+        const pdf_int64 lOff     = tokenizer.GetNextNumber();
         const std::streamoff pos = device.Device()->Tell();
 
         // move to the position of the object in the stream
@@ -87,7 +100,14 @@ void PdfObjectStreamParserObject::ReadObjectsFromStream( char* pBuffer, pdf_long
 
 		// use a second tokenizer here so that anything that gets dequeued isn't left in the tokenizer that reads the offsets and lengths
 	    PdfTokenizer variantTokenizer( device, m_buffer );
-        variantTokenizer.GetNextVariant( var, m_pEncrypt );
+		if( m_pEncrypt && (m_pEncrypt->GetEncryptAlgorithm() == PdfEncrypt::ePdfEncryptAlgorithm_AESV2
+#ifndef OPENSSL_NO_RC4
+			|| m_pEncrypt->GetEncryptAlgorithm() == PdfEncrypt::ePdfEncryptAlgorithm_RC4V2 
+#endif // OPENSSL_NO_RC4
+			                                                                              ) )
+			variantTokenizer.GetNextVariant( var, 0 ); // Stream is already decrypted
+		else
+			variantTokenizer.GetNextVariant( var, m_pEncrypt );
 		bool should_read = std::find(list.begin(), list.end(), lObj) != list.end();
 #if defined(PODOFO_VERBOSE_DEBUG)
         std::cerr << "ReadObjectsFromStream STREAM=" << m_pParser->Reference().ToString() <<
@@ -96,12 +116,12 @@ void PdfObjectStreamParserObject::ReadObjectsFromStream( char* pBuffer, pdf_long
 #endif
 		if (should_read)
         {
-			if(m_vecObjects->GetObject(PdfReference( static_cast<int>(lObj), 0LL ))) 
+			if(m_vecObjects->GetObject(PdfReference( static_cast<int>(lObj), PODOFO_LL_LITERAL(0) ))) 
             {
                 PdfError::LogMessage( eLogSeverity_Warning, "Object: %li 0 R will be deleted and loaded again.\n", lObj );
-                delete m_vecObjects->RemoveObject(PdfReference( static_cast<int>(lObj), 0LL ),false);
+                delete m_vecObjects->RemoveObject(PdfReference( static_cast<int>(lObj), PODOFO_LL_LITERAL(0) ),false);
             }
-            m_vecObjects->insert_sorted( new PdfObject( PdfReference( static_cast<int>(lObj), 0LL ), var ) );
+            m_vecObjects->insert_sorted( new PdfObject( PdfReference( static_cast<int>(lObj), PODOFO_LL_LITERAL(0) ), var ) );
 		}
 
         // move back to the position inside of the table of contents
